
/*****************************************************************************
Programs must be added to the four places below:

Command name
Command 
Do Command
Manual Page
add registers to all functions
use registers all functions


FIX failed Operand freezing (no operand crashes) must fix bad input crashes on all functions
SEND TO SPEECH PROCESSOR
FORMAT WITH ALEX-TABS
CONTROL FET
PWM OUT
HIGH LOW TEST
LED PULSE OUT
TEXT TO SPEECH DUH!

RADIATION random number generator
SET range of numbers

CRASHES AFTER RADNUM

Make TRNG pull from stack by default
TRNG seed is wierd put in trng 20 and it counts from 0 - 7

see if I can free strings from memory

Alarms and Timers Set and Clear
GPS time getter with automatic timezone option

With the GPS time get a cycles count per micro second number to tune the cycles to atomic time.

timezone alarm
speed
time
direction
location
automatic time zone change
elevation
signal strength / sattalite count
gps subsystem power management control
jamming interference indication
spoofing detection
GNSS satalite selection settings 
gps subsystem reset, 3 types cold, warm, hot


https://www.pjrc.com/teensy/td_libs_Time.html has GPS Example code.

   
   Simple and Clear Command Line Interpreter

     This file will allow you to type commands into the Serial Window like,
        add 23,599
        blink 5
        playSong Yesterday

   1)  Simple Protocol
         Commands are words and numbers either space or comma spearated
         The first word is the command, each additional word is an argument
         "\n" terminates each command

   

**********************************************************************************/

//Proprocessor Directives
#include <TimeLib.h>                //For Human Time Keeping
//#include <TimeAlarms.h>             //For Human Alarms
//#include "Alarms.h"                 //Custom Alarm Methods

#include "rando.h"                  //TRNG On-Board Processor - Self Seeded
#include "Rad_Count.h"              //Radiation Counter for Superposition Random Number and Sv/h Counter
#include "Sorters.h"                //Sorting Algorithms for Sorting and Searching Tests

#include <string.h>
#include <stdlib.h>
#include <SoftwareSerial.h>         //For 2nd serial bus, full duplex communication with EMIC Speech Module
#include "manual.h"                 //For Man pages

#define ledPin  13                  // On-board Debug and Status LED on Teensy 
#define rxPin   7                   // Serial Teensy input (to Emic 2's SOUT)
#define txPin   8                   // Serial Teensy output (to Emic 2's SIN)


//Global Variables

//extern float tempmonGetTemp(void);  //CPU Temperature Built in on Teensy 4.0

static float firstOperand = 0;      //Global Math Variables for function operations
static float secondOperand = 0;

static short int indexa = 0;
static short int indexb = indexa +1;
static int* group;

static int groupForward[200];       //Arrays for Sorting and Searching Tests - Integers Charged Ascending 
static int groupBack[200];          //Arrays for Sorting and Searching Tests - Integers Charged Descending
static int groupRandom[200];        //Arrays for Sorting and Searching Tests - Integers Charged Randomly

//Return Stack
static float primaryRegister  = 0.0;
static float nextRegister     = 0.0;

//Function / Method Prototypes
FLASHMEM int Alexort(void);
FASTRUN short unsigned int init_EMIC();

void ChargeGroupArrays(void);
void PrintGroupArrays(void);


// New secondary serial port for EMIC Processor Bus
SoftwareSerial emicSerial =  SoftwareSerial(rxPin, txPin);


//this following macro is good for debugging, e.g.  print2("myVar= ", myVar);
#define print2(x,y) (Serial.print(x), Serial.println(y))


#define CR '\r'
#define LF '\n'
#define BS '\b'
#define NULLCHAR '\0'
#define SPACE ' '

#define COMMAND_BUFFER_LENGTH        25                        //length of serial buffer for incoming commands
char   CommandLine[COMMAND_BUFFER_LENGTH + 1];                 //Read commands into this buffer from Serial.  +1 in length for a termination char

const char *delimiters            = ", \n";                    //commands can be separated by return, space or comma



/*************************************************************************************************************
     your Command Names Here
*/
PROGMEM const char *addCommandToken         = "add";                        //Add from Flash
PROGMEM const char *subtractCommandToken    = "sub";                        //Subtract from Flash
PROGMEM const char *multiplyCommandToken    = "mult";                       //Multiply from Flash
PROGMEM const char *divideCommandToken      = "div";                        //Divide from Flash

PROGMEM const char *init_EMICCommandToken   = "initspeak";                  //Initialize Speech Processor
PROGMEM const char *sayhelloToken           = "sayhi";                      //Test Speech Processor
PROGMEM const char *sayVarsToken            = "say";                        //Output to Speech Processor

//PROGMEM const char *getTempToken            = "gettemp";                    //Get Cortex CPU Temperature
PROGMEM const char *cpuTestToken            = "cputest";                    //Timed Alexort CPU Charge and Sort Test
                                                                            //From High-Speed Memory (Cache)
PROGMEM const char *ledToggleToken          = "ledt";                       //Toggle state or command on or off

PROGMEM const char *manPageToken            = "man";                        //short int manPage(void) manual.h 
PROGMEM const char *helpPageToken           = "help";                       //short int manPage(void) manual.h 

PROGMEM const char *setTimeToken            = "sett";                       //short int settime(void) manual.h 
PROGMEM const char *getTimeCommandToken     = "gett";                       //Get time from Cortex

PROGMEM const char *seeStackToken           = "cstack";                     //View Stack Registers
PROGMEM const char *pushStackToken          = "pstack";                     //Push Stack Registers

PROGMEM const char *trngkToken              = "trng";                       //Get Random Number - Hardware
PROGMEM const char *radNumToken             = "radnum";                     //Get Random Number - "Superposition" Radiation
PROGMEM const char *radCountToken           = "radcount";                   //Get Radiation Count in uSv/h

/*************************************************************************************************************
    getCommandLineFromSerialPort()
      Return the string of the next command. Commands are delimited by return"
      Handle BackSpace character
      Make all chars lowercase
*************************************************************************************************************/

bool
getCommandLineFromSerialPort(char * commandLine)
{
  static uint8_t charsRead = 0;                      //note: COMAND_BUFFER_LENGTH must be less than 255 chars long
  //read asynchronously until full command input
  while (Serial.available()) {
    char c = Serial.read();
    switch (c) {
      case CR:      //likely have full command in buffer now, commands are terminated by CR and/or LS
      case LF:
        commandLine[charsRead] = NULLCHAR;       //null terminate our command char array
        if (charsRead > 0)  {
          charsRead = 0;                           //charsRead is static, so have to reset
          Serial.println(commandLine);
          return true;
        }
        break;
      case BS:                                    // handle backspace in input: put a space in last char
        if (charsRead > 0) {                        //and adjust commandLine and charsRead
          commandLine[--charsRead] = NULLCHAR;
          Serial << byte(BS) << byte(SPACE) << byte(BS);  //no idea how this works, found it on the Internet
        }
        break;
      default:
         c = tolower(c);                                  //Case Sensitivity flag
        if (charsRead < COMMAND_BUFFER_LENGTH) {
          commandLine[charsRead++] = c;
        }
        commandLine[charsRead] = NULLCHAR;     //just in case
        break;
    }
  }
  return false;
}


/* ****************************
   readNumber: return a 16bit (for Arduino Uno) signed integer from the command line
   readWord: get a text word from the command line

*/
float readNumber ()                 
  {
    char * numTextPtr = strtok(NULL, delimiters);         //K&R string.h  pg. 250

    
    
    return atof(numTextPtr);                              //K&R string.h  pg. 251  //was atoi
  }

char * readWord() {
  char * word = strtok(NULL, delimiters);               //K&R string.h  pg. 250
  return word;
}

void
nullCommand(char * ptrToCommandName) {
  print2(" Type 'man'. Command not found: ", ptrToCommandName);      //see above for macro print2
}


/****************************************************
   Add your commands here
*/

void formatDigits(int digits)                           //Required for terminal time output formating
  {
    if (digits < 10)
      {
        Serial.print('0');    
      }
      
  Serial.print(digits);
  }


float gettime(String Operand)
  {
    if(Operand == "all")
      {
        Serial.println("");
        Serial.print("Time is Set to: ");
        formatDigits(hour());
        Serial.print(":");
        formatDigits(minute());
        Serial.print(":");
        formatDigits(second());
        Serial.println("");
        Serial.println("-----------------------------------------");

        gettime("date");
        
        primaryRegister = second();
        nextRegister = minute();
        return hour();
        
      }
    
    else if(Operand == "date")
      {
        Serial.println("Date: ");
        Serial.println("");
        Serial.print("Month: ");
        formatDigits(month());
        Serial.println("");
        Serial.print("Day: ");
        formatDigits(day());
        Serial.println("");
        Serial.print("Year: ");
        formatDigits(year());
        Serial.println("");

        primaryRegister = month();
        nextRegister = day();
        return year();
      }
    else if(Operand == "minute")
      {
        Serial.print(":");
        formatDigits(minute());
        Serial.println(":");

        nextRegister = primaryRegister;
        primaryRegister = minute();
        return primaryRegister;
      }
    else if(Operand == "year")
      {
        Serial.print("Year: ");
        formatDigits(year());
        Serial.println("");

        nextRegister = primaryRegister;
        primaryRegister = year();
        return primaryRegister;
      }
    else if(Operand == "month")
      {
        Serial.print("Month: ");
        formatDigits(month());
        Serial.println("");

        nextRegister = primaryRegister;
        primaryRegister = month();
        return primaryRegister;
      }
    else if(Operand == "day")
      {
        Serial.print("Day: ");
        formatDigits(day());
        Serial.println("");

        nextRegister = primaryRegister;
        primaryRegister = day();
        return primaryRegister;
      }
    else if (Operand == "hour")
      {
        Serial.print(":");
        formatDigits(hour());
        Serial.println(":");

        nextRegister = primaryRegister;
        primaryRegister = hour();
        return primaryRegister; 
      }
    else if (Operand == "second")
      {
        Serial.print(":");
        formatDigits(second());
        Serial.println(":");

        nextRegister = primaryRegister;
        primaryRegister = second();
        return second();
      }
     else if (Operand == "th")
      {
        Operand = readWord();

        primaryRegister = hour();
        String daynight;
        
        if(primaryRegister > 12)
          {
            primaryRegister = primaryRegister - 12;
            nextRegister = 1;
            daynight = "pm"; 
          }
        else
          {
            daynight = "am";
            nextRegister = 0;
          }
        
        if(Operand == "h")
          {
            formatDigits(primaryRegister);
            Serial.println(daynight);
            return primaryRegister;  
          }
        else
          {
            Serial.println("-----------------------------------------");
            Serial.print("12 Hour Time: ");
            formatDigits(primaryRegister);
            Serial.print(":");
            formatDigits(minute());
            Serial.print(":");
            formatDigits(second());
            Serial.println("");
            Serial.println("-----------------------------------------");
            return primaryRegister; 
          }
      }
     else
      {
        Serial.println("-----------------------------------------");
        Serial.print("Time is Set to: ");
        formatDigits(hour());
        Serial.print(":");
        formatDigits(minute());
        Serial.print(":");
        formatDigits(second());
        Serial.println("");
        Serial.println("-----------------------------------------");
        primaryRegister = second();
        nextRegister = minute();
        return hour();
      }

    return 999.999;
  }//END gettime()



FLASHMEM float settime()
  {
    firstOperand = readNumber();                  //..Grab first two numbers
    secondOperand = readNumber();                 //Hours Minutes
    unsigned short getseconds = readNumber();     //Seconds
    unsigned short getmonth   = readNumber();     //Month
    unsigned short getday     = readNumber();     //Day
    unsigned short getyear    = readNumber();     //Year

    setTime(firstOperand, secondOperand, getseconds, getday, getmonth, getyear);

    gettime("all");

    primaryRegister = hour();
    nextRegister = minute();
    return second();
  }//END settime()


FLASHMEM float addCommand()
  {                                      //Modify here
    firstOperand = readNumber();
    secondOperand = readNumber();

    nextRegister = primaryRegister;
    primaryRegister = firstOperand + secondOperand;
    return primaryRegister;
  }

FLASHMEM float subtractCommand()
  {                                //Modify here
    firstOperand = readNumber();
    secondOperand = readNumber();
    
    nextRegister = primaryRegister;
    primaryRegister = firstOperand - secondOperand;
    return primaryRegister;
  }

FLASHMEM float multiplyCommand()
  {
    firstOperand = readNumber();
    secondOperand = readNumber();
    nextRegister = primaryRegister;
    
    primaryRegister = firstOperand * secondOperand;
    return primaryRegister;
  }

FLASHMEM float divideCommand()
  {
    firstOperand = readNumber();
    secondOperand = readNumber();
    
    nextRegister = primaryRegister;
    primaryRegister = firstOperand / secondOperand;
    return primaryRegister;
  }


FLASHMEM float getTemp()
  {
    String Operand = readWord();            //Get Command Modifier String after initial command
    
    if(Operand == "c")
      {
        //Serial.print( tempmonGetTemp() );
        Serial.println("°C");

        nextRegister = primaryRegister;
        //primaryRegister = tempmonGetTemp();
        return primaryRegister;
        
      }
    else if(Operand == "f")
      {
        nextRegister = primaryRegister;
        //primaryRegister =  (tempmonGetTemp() * 9.0 / 5.0) + 32.0 ;   //C to F Temperature
        //Serial.print(primaryRegister);
        Serial.println("°F");
        return primaryRegister;
      }
    else
      {
        Serial.println(" ADD 'C' or 'F' modifier to command");
        
        nextRegister = primaryRegister;
        //primaryRegister = tempmonGetTemp();
        Serial.println(Operand);
        return primaryRegister;
      }
      
  }//END getTemp()

FASTRUN short unsigned int sayVars()
  {
    String Operand = readWord();            //Get Command Modifier String after initial command
    if(Operand == "S")
      {
        init_EMIC();
        
        emicSerial.print('S');
        emicSerial.print(primaryRegister);  // string to convert to speech
        emicSerial.print('\n');
        return primaryRegister;
      }
    else if (Operand == "D")
      {
        init_EMIC();
    
        emicSerial.print('S');
        emicSerial.print(nextRegister);  // string to convert to speech
        emicSerial.print('\n');
        return nextRegister;
      }
     else
      {
        return primaryRegister;
      }
      
  }//END sayVars() Method

FASTRUN short unsigned int init_EMIC()
  {
    emicSerial.begin(9600);             // set data rate for the SoftwareSerial port (EMIC bus)
    delay(20);
    emicSerial.print('\n');             // Send a CR in case the system is already up

    digitalWrite(ledPin, LOW);      // Status LED OFF
    delay(10); 
    for(short int index = 0;index < 100; index++)
      {
        if (emicSerial.read() == ':')               //Char is sent when EMIC is ready and initialized
          {
            Serial.println("EMIC Speech Processor Initialized..");
            delay(10);                             // Short delay
            emicSerial.flush();                    // Flush the receive buffer
            digitalWrite(ledPin, HIGH);            // Status LED ON
            nextRegister = primaryRegister;
            primaryRegister = 1;
            return primaryRegister;
          }
        else
          {
            Serial.print("Connection Attempt: ");
            Serial.print(index + 1);
            Serial.println(" Failed.");
            delay(20);
            emicSerial.print('\n');               // Send a CR in case the system is already up
          }
        delay(20);
      }
  
    nextRegister = primaryRegister;
    primaryRegister = 0;
    return primaryRegister;
  }//END init_EMIC() Function

FASTRUN short unsigned int sayhello()
  {
    init_EMIC();
    
    emicSerial.print('S');
    emicSerial.print("Hello. If you can hear this, the speech Processor is on-line.");  // string to convert to speech
    emicSerial.print('\n');
    nextRegister = primaryRegister;
    primaryRegister = 1;
    return primaryRegister;
  }

//FASTRUN uint64_t get_RTC_periods()                     //Teensy Timing Function to get a periods epoch
//{
////    uint32_t hi1 = SNVS_HPRTCMR, lo1 = SNVS_HPRTCLR;   
//    while (true)
//    {
////        uint32_t hi2 = SNVS_HPRTCMR, lo2 = SNVS_HPRTCLR; 
////        if (lo1 == lo2 && hi1 == hi2)
////        {
////            return (uint64_t)hi2 << 32 | lo2;
////        }
//        hi1 = hi2;
//        lo1 = lo2;
//    }
//}



FASTRUN short int TimedAlexort()
  {
    Serial.println("Working...");
//   uint64_t startPeriod = get_RTC_periods();

    firstOperand = readNumber();

    if(firstOperand > 3000 || firstOperand < 1)
      {
        firstOperand = 100;
      }

    if(firstOperand < 1 || firstOperand > 30000)
      {
        firstOperand = 100;                  
      }
      
    group = (int*)malloc(firstOperand * sizeof(int));   //Dynamically Allocate Array

    if (group == NULL)
      {
        Serial.println("Memory not allocated.");
        
        nextRegister = primaryRegister;
        primaryRegister = 0;
        return primaryRegister;
      }

    
    Serial.println("Working... Charging Array...");
    //Charging Array
    for(int i = 0 ; i < firstOperand; i++)
      {
        group[i] = i + 1;
        Serial.print(group[i]);

        if(i % 30 == 0)                //Format Serial Terminal Output
          {
            Serial.println("");
          }
      }//END FOR Charging array
    
    Serial.println("");
    Serial.println("Working... Recursive Sorting...");
      
    //RECURSIVE ALEXORT() WRAPPED HERE
    Alexort();
    //END OF RECURSIVE ALEXORT()
    
    Serial.println("Working... Displaying Array");
   //Display Array
    for(int i = 0 ; i < firstOperand; i++)
      {
        Serial.print(group[i]);

        if(i % 30 == 0)                //Format Serial Terminal Output
          {
            Serial.println("");
          }
      }//END FOR Display Array  
    
//    uint64_t passedPeriods = get_RTC_periods() - startPeriod;
    Serial.println("");
    Serial.print("CPU Test Successfully Completed In Seconds: ");
//    Serial.println(passedPeriods / 32768);
    Serial.print("Periods: ");
//    Serial.println(passedPeriods);

    free(group);                     //Deallocate Memory for dynamic array: group

    nextRegister = primaryRegister;
    primaryRegister = 1;
    return primaryRegister;
  }


FLASHMEM short int ledt(void)
  {
    String Operand = readWord();            //Get Command Modifier String after initial command
    
    if(Operand == "ON")
      {
        digitalWrite(ledPin, HIGH);
      }
    else if (Operand == "OFF")
      {
        digitalWrite(ledPin, LOW);
      }
     else
      {
        digitalWrite(ledPin, !digitalRead(ledPin));
      }

      nextRegister = primaryRegister;
      primaryRegister = float(digitalRead(ledPin));
      return digitalRead(ledPin);
  }



/****************************************************
   DoMyCommand
*/
bool
DoMyCommand(char * commandLine)
{
  //  print2("\nCommand: ", commandLine);
  float result;                               //Was int

  char * ptrToCommandName = strtok(commandLine, delimiters);
  //  print2("commandName= ", ptrToCommandName);

  if (strcmp(ptrToCommandName, addCommandToken) == 0)
    {                   //Modify here
      result = addCommand();
      print2(">    The sum is = ", result);
    }

  else if (strcmp(ptrToCommandName, getTimeCommandToken) == 0)
      {                   
        String Operand = readWord();            //Get Command Modifier String after initial command
        result = gettime(Operand);
        print2(">    Return = ", result);
      }
      
  else if (strcmp(ptrToCommandName, sayVarsToken) == 0)
      {                   
        String Operand = readWord();            //Get Command Modifier String after initial command
        result = sayVars();
        print2(">    Return = ", result);
      }

  else if (strcmp(ptrToCommandName, init_EMICCommandToken) == 0)
      {                   //Modify here
        result = init_EMIC();
        print2(">    Status = ", result);
      }

  else if (strcmp(ptrToCommandName, sayhelloToken) == 0)
      {                   //Modify here
        result = sayhello();
        print2(">    Function Execution = ", result);
      }
   
//  else if (strcmp(ptrToCommandName, getTempToken) == 0)
//      {                   
//        result = getTemp();
//        print2(">    Return = ", result);
//      }

   else if (strcmp(ptrToCommandName, divideCommandToken) == 0)
      {                   
        result = divideCommand();
        print2(">    The quotient is = ", result);
      }

  else if (strcmp(ptrToCommandName, multiplyCommandToken) == 0)
      {                   
        result = multiplyCommand();
        print2(">    The product is = ", result);
      }

  else if (strcmp(ptrToCommandName, cpuTestToken) == 0)
      {  
        nextRegister = primaryRegister;
//        trng_init();
        
        Serial.println("");
        Serial.println("Charging Arrays...");
        ChargeGroupArrays();
        Serial.println("Arrays have been initialised and Charged.");
        Serial.println("");

        Serial.println("Applying Bubble Sort.");
        bubbleSort(groupForward,200);   //Return times needs to be added
        bubbleSort(groupBack,200);
        bubbleSort(groupRandom,200);

        Serial.println("Printing Arrays...");
        PrintGroupArrays();

        Serial.println("Charging Arrays...");
        ChargeGroupArrays();

        Serial.println("");
        Serial.println("Applying Selection Sort.");
        selectionSort(groupForward,200);
        selectionSort(groupBack,200);
        selectionSort(groupRandom,200);

        Serial.println("Printing Arrays...");
        PrintGroupArrays();


          
        result = TimedAlexort();
        print2(">    Return = ", result);
      }

  else if (strcmp(ptrToCommandName, ledToggleToken) == 0)
      {                   
        result = ledt();
        print2(">    Status = ", result);
      }

  else if (strcmp(ptrToCommandName, manPageToken) == 0 || strcmp(ptrToCommandName, helpPageToken) == 0)
      {                   
        String Operand = readWord();            //Get Command Modifier String after initial command
        String Modif = readWord();              //Get Modifiers
        String Modif2 = readWord();
        result = manPage(Operand, Modif, Modif2);
        print2(">    Number of Entries = ", result);
      }
      
  else if (strcmp(ptrToCommandName, setTimeToken) == 0)
      {                   
        result = settime();
        print2(">    Time Set OK = ", result);
      }

  else if (strcmp(ptrToCommandName, seeStackToken) == 0)
      {                   
        Serial.print("Stack Value: ");
        Serial.println(primaryRegister);
        Serial.print("Deep Stack: ");
        Serial.println(nextRegister);
      }
  else if (strcmp(ptrToCommandName, pushStackToken) == 0)
      {
        nextRegister = primaryRegister;
        primaryRegister = readNumber();
        Serial.println("OK");
      }

  else if (strcmp(ptrToCommandName, trngkToken) == 0)
      {
        nextRegister = primaryRegister;
//        trng_init();
        //initTRNG();
        primaryRegister = readNumber();
        if(primaryRegister >= 16)
          {
            Serial.println("Seed must be lower than 16. Defaulting to 1");
            primaryRegister = 1;
          }
        primaryRegister = float(getRand(primaryRegister));
        
      }
  else if (strcmp(ptrToCommandName, radNumToken) == 0)
      {
        nextRegister = primaryRegister;
        primaryRegister = RadNum();
        
      }
  else if (strcmp(ptrToCommandName, radCountToken) == 0)
      {
        nextRegister = primaryRegister;
        firstOperand = readNumber();
        if(firstOperand > 0 && firstOperand < 1000)
          {
            primaryRegister = RadCount(int(firstOperand));
          }
        else
          {
            Serial.println("Sample count must be between 1 and 999.");
            Serial.println("Warning! Large sample counts may take considerable time.");
            Serial.println("This counter has a gamma source close to the emitter, so counts will be relative.");
          }
      }
  else
    {
        if (strcmp(ptrToCommandName, subtractCommandToken) == 0)
          {           //Modify here
            result = subtractCommand();                                       //K&R string.h  pg. 251
            print2(">    The difference is = ", result);
          }

      else
        {
          nullCommand(ptrToCommandName);
        }
    }
}


FLASHMEM int Alexort(void)
  {
    if(indexa >= firstOperand - 1)
      {
        return 1;
      }

    if(group[indexa] < group[indexb])
      {
        group[indexa] = group[indexa] + group[indexb];
        group[indexb] = group[indexa] - group[indexb];
        group[indexa] = group[indexa] - group[indexb];
      }
    
    indexb += 1;

    if(indexb >= firstOperand)
      {
        indexa += 1;
        indexb = indexa + 1;
      }

    Alexort();

}//END Alexort() Function



void ChargeGroupArrays(void)
  {
    for(short int index = 0;index < 200; index++)
      {
        Serial.println("");
        groupForward[index] = index;
        Serial.print("Forward: ");
        Serial.print(groupForward[index]);
        groupBack[index]    = 199 - index;
        Serial.print(" Back: ");
        Serial.print(groupBack[index]);
        Serial.print(" Random: ");
        groupRandom[index] = int(getRand(index % 16));      //Function will Terminal-Out on it's own   
      }
  }//END ChargeGroupArrays Function

void PrintGroupArrays(void)
  {
    for(int index = 0; index < 200; index++)
      {
        Serial.println("");
        Serial.print("Forward: ");
        Serial.print(groupForward[index]);
        Serial.print(" Back: ");
        Serial.print(groupBack[index]);
        Serial.print(" Random: ");
        Serial.print(groupRandom[index]);
      }
  }
  
