
//Preprocessor Directives
#include <RadiationWatch.h>



RadiationWatch radiationWatch;

//Prototypes
void ChargeEntropy(int startcell);
void startEntropy();
void onNoise();
void onRadiation();
void resetEntropy();

//Global Variables
bool bitMatrix[2][8];         //Quantom voting matrix by Uranium 238 decay Must start at both 1 and 0 to vote against
bool collapsedEntropy[8];     //Result of total entropy
bool gammaRad = false;        //Once a gamma ray is caught, flip the bit TRUE;

byte RandomNumber = 0;        //Solidified variable to bitshift result into

//Radiation Counter Setup
void setupRad()
  {
    radiationWatch.setup();
    radiationWatch.registerRadiationCallback(&onRadiation);
    radiationWatch.registerNoiseCallback(&onNoise);
  }

void onRadiation()
  {
    gammaRad = true;
  }

void onNoise()
{
  Serial.println("Vibration Detected. Keep Unit still..");
}

void ChargeEntropy(int startcell)  //Start at 0
  {
   
    
        while(!gammaRad)
          {
            radiationWatch.loop();
            bitMatrix[0][startcell] = !bitMatrix[0][startcell];     //Flip the bit
          }
          
        gammaRad = false;
        
        while(!gammaRad)
          {
            radiationWatch.loop();
            bitMatrix[1][startcell] = !bitMatrix[1][startcell];     //Flip the bit
          } 

        gammaRad = false;

        

        if(bitMatrix[0][startcell] == bitMatrix[1][startcell])
        {
          collapsedEntropy[startcell] = bitMatrix[0][startcell];
          startcell++;
          
        }

        if(startcell < 8)
        {
          ChargeEntropy(startcell);
        }
        
  }//END ChargeEntropy(int startCell)


int RadNum()                                //Entry point for Random Number Operation
  {
    for(int index = 0; index < 32; index++)
      {
        setupRad();
        resetEntropy();
        startEntropy();
        Serial.print(RandomNumber);
        
      }
      
    Serial.println("");
    return RandomNumber;
  }


float RadCount(int samplesize)                            //Entry point for uSv/Hour Counter
  {
    float count = 0;
    if(samplesize > 10)
      {
        Serial.println(" -WARNING- Large sample sizes may take a very long time, depending on ");
        Serial.println("the level of background radiation.");
      }
    for(int index = 0; index < samplesize;)
      {
        while(!gammaRad)
          {
            radiationWatch.loop();
          }         
        Serial.println("Gamma Radiation Detected");
        count = radiationWatch.uSvh();
        Serial.print(count);
        Serial.print(" uSv/h within a +/- ");
        Serial.print(radiationWatch.uSvhError());
        Serial.println(" margin of error...");
        gammaRad = false;
        index++;
      }//END FOR
    Serial.println("Done.");
    return count;
  }//END int RadCount()


 void resetEntropy()                            //Reset Vote to 1's and 0's to remove stain of test on universe
  {
    for(int index = 0;index < 8; index++)
      {
        bitMatrix[0][index] = 1; 
      }
    for(int index = 0;index < 8; index++)
      {
        bitMatrix[1][index] = 0; 
      }
  }//END resetEntropy()


void startEntropy()
  {
    ChargeEntropy(0);
    RandomNumber = 0;
    for(int index = 7; index > -1; index--)
      {
        //Serial.print(collapsedEntropy[index]);
        RandomNumber += collapsedEntropy[index] << index;
      }
    //Serial.println("");
  }
